import random

from Products.CMFCore.utils import getToolByName

import tabellio.config.utils

class BasePublication(object):
    @property
    def date_str(self):
        return self.date.strftime('%d/%m/%Y')

    def get_authors_as_string(self):
        if not self.authors:
            return ''
        return ', '.join([x.to_object.title for x in self.authors])

    @property
    def authors_html_str(self):
        if not self.authors:
            return ''
        return ', '.join([u'<a href="%s">%s</a>' % (x.to_object.absolute_url(), x.to_object.title)
                          for x in self.authors])

    @property
    def topic_titles(self):
        from plone.i18n.normalizer.fr import normalizer
        topic_dict = tabellio.config.utils.get_topics_dict()
        def cmp_topic(x, y):
            return cmp(normalizer.normalize(x).lower(),
                       normalizer.normalize(y).lower())
        return sorted([topic_dict.get(x, [x])[0] for x in self.topics], cmp_topic)

    @property
    def polgroups_str(self):
        if not self.polgroups:
            return ''
        return ', '.join([x.to_object.title for x in self.polgroups])

    def related_elements(self, max_items=5, closeness=0.5):
        if not self.topics:
            return []

        topics = self.topics[:]
        # try to get other elements sharing the same topics
        catalog = getToolByName(self, 'portal_catalog')
        previous_result = None
        while len(topics) > len(self.topics)*closeness:
            brains = catalog(portal_type=self.portal_type,
                             topics={'query': topics, 'operator': 'and'})
            if len(brains) > max_items:
                brains = list(brains)
                random.shuffle(brains)
                if previous_result:
                    brains = list(previous_result) + brains
                return brains[:max_items]
            elif len(brains) > max_items/2:
                return brains
            previous_result = brains
            del topics[random.randint(0, len(topics)-1)]
        return []
